// Forlix SrcDS LogMerge
// http://forlix.org/, df@forlix.org
//
// Copyright (c) 2008-2011 Dominik Friedrichs

#include "SrcDS_LogMerge.h"


int MergeLogList(char list[][SORTSTR_MAXLEN],
                 size_t list_len)
{
  FILE *hmlog = NULL;

  time_t time_c;
  time(&time_c);

  tm ct;
  ct = *(localtime(&time_c));

  int mlog_year = 0;
  int mlog_month = 0;

  char mlogfile[16];
  unsigned int mergedcnt = 0;

  for(unsigned int i = 0; i < list_len; i++)
  {
    FILE *hlog = NULL;
    char logfile[16];

    char line[LINE_MAXLEN];
    char logstr[LOGSTR_MAXLEN];

    time_t time_l;
    time_t time_m;
    tm lt;

    if(sscanf(list[i], "%010u_%[^\n]s", &time_l, logfile) != 2
    || (hlog = fopen(logfile, "rt")) == NULL)
    // extraction of timestamp and logfile name failed
    // or failed to open logfile
      continue;

    lt = *(localtime(&time_l));

    if(lt.tm_year+1900 != mlog_year
    || lt.tm_mon+1 != mlog_month)
    // current source log is not from same month as what we opened mergedlog for
    // so we need to start a new mergedlog for the month in question
    {
      if(hmlog)
      {
        fclose(hmlog);
        hmlog = NULL;

        if(mergedcnt)
          printf("%u logs to %s\n\n", mergedcnt, mlogfile);
        else
          printf("No logs to %s\n\n", mlogfile);
      }

      mlog_year = lt.tm_year+1900;
      mlog_month = lt.tm_mon+1;

      sprintf(mlogfile, "%04u%02u.log", mlog_year, mlog_month);

      if(!(hmlog = fopen(mlogfile, "at+")))
      {
        printf("Fatal error when trying to write %s\n", mlogfile);
        return(0);
      }

      printf("Merging...\n");
    }

    // get the timestamp of mergedlog's last line
    fgetsl(line, sizeof(line), hmlog);
    fseek(hmlog, 0, SEEK_END);

    tm tmerged;
    if(ReadLogLine(line, &tmerged, logstr))
      time_m = mktime(&tmerged);
    else
    // couldnt verify mergedlog's last line, so its probably empty
      time_m = 0;

    if(ct.tm_mday == lt.tm_mday
    && ct.tm_mon == lt.tm_mon
    && ct.tm_year == lt.tm_year)
    // dont merge files dated today, it will make srcds start the next log at lMMDD000.log again
    {
      printf("%s omitted (dated today)\n", logfile);
      fclose(hlog);

      continue;
    }

    if(time_l < time_m)
    // logfile timestamp dated earlier than the last line of the merged log
    {
      printf("%s omitted (dated earlier than end of merged log)\n", logfile);
      fclose(hlog);

      continue;
    }

    if(i == list_len-1)
    // most recent file - we need to check if it was terminated
    {
      line[0] = '\0';

      fgetsl(line, sizeof(line), hlog);
      fseek(hlog, 0, SEEK_SET);

      if(!ReadLogLine(line, &lt, logstr)
      || !strpequ(logstr, "Log file closed"))
      // last line of most recent log is not terminated - omit merging this file for now
      {
        printf("%s omitted (not yet terminated)\n", logfile);
        fclose(hlog);

        continue;
      }
    }

    // transfer the log's lines to the mergedlog
    line[0] = '\0';

    for(fgets(line, sizeof(line), hlog);
        feof(hlog) == 0;
        fgets(line, sizeof(line), hlog))
      fprintf(hmlog, "%s", line);

    fflush(hmlog);
    mergedcnt++;

    fclose(hlog);

    if(remove(logfile))
      printf("Error deleting %s\n", logfile);
  }

  if(hmlog)
  {
    fclose(hmlog);
    hmlog = NULL;

    if(mergedcnt)
      printf("%u logs to %s\n\n", mergedcnt, mlogfile);
    else
      printf("No logs to %s\n\n", mlogfile);
  }

  return(1);
}
