// Forlix SrcDS LogMerge
// http://forlix.org/, df@forlix.org
//
// Copyright (c) 2008-2011 Dominik Friedrichs

#include "SrcDS_LogMerge.h"


// strequ - string testing for equality

bool strequ(const char *str1,
            const char *str2,
            bool case_sens)
{
  if(!str1 || !str2)
    return(false);

  if(case_sens)
    return(strcmp(str1, str2) == 0);
  else
    return(stricmp(str1, str2) == 0);
}


// strpequ - partial string testing for equality
// returns true if str begins with ref

bool strpequ(const char *str,
             const char *ref)
{
  if(!str || !ref)
    return(false);

  while(*ref)
    if(*str++ != *ref++)
      return(false);

  return(true);
}


// fgetsl - reads the last
// \n-terminated line of a textfile

char *fgetsl(char *str,
             int num,
             FILE *fh)
{
  int p = num-1;
  char readchr;
  fseek(fh, -1, SEEK_END);

  do
  {
    readchr = fgetc(fh);
    fseek(fh, -2, SEEK_CUR);
  }
  while(readchr != '\n'
     && readchr != EOF
     && p >= 0);

  fseek(fh, -1, SEEK_CUR);
  str[p--] = '\0';

  if(readchr == '\n')
  {
    str[p--] = '\n';

    do
    {
      str[p--] = fgetc(fh);
      fseek(fh, -2, SEEK_CUR);
    }
    while(str[p+1] != '\n'
       && str[p+1] != EOF
       && p >= 0);

    if(str[p+1] == '\n'
    || str[p+1] == EOF)
    {
      p += 2;

      for(int q = 0; q < num-p; q++)
        str[q] = str[p+q];
    }
  }
  else
    str[0] = '\0';

  fseek(fh, 2, SEEK_CUR);
  return(str);
}


bool ReadLogLine(const char *line,
                 tm *logtime,
                 char *logstr)
{
  if(sscanf(line, "L %02u/%02u/%04u - %02u:%02u:%02u: %[^\n]s",
                  &logtime->tm_mon, &logtime->tm_mday, &logtime->tm_year,
                  &logtime->tm_hour, &logtime->tm_min, &logtime->tm_sec, logstr) != 7)
  // line not successfully read
    return(false);

  logtime->tm_year -= 1900;
  logtime->tm_mon -= 1;

  logtime->tm_wday = -1;
  logtime->tm_yday = -1;
  logtime->tm_isdst = -1;
  return(true);
}


int loglist_sortcomp(const void *a,
                     const void *b)
{
  return(strcmp((const char *)a,
                (const char *)b));
}
